//
// A compiler par gcc -O3 myprog.c -o hi -lX11 -lXt -lXaw
//
// $Id: myprog.c,v 1.8 2005/05/20 23:10:24 kenjen Exp $
//
// $Log: myprog.c,v $
// Revision 1.8  2005/05/20 23:10:24  kenjen
// Determined the correct interaction needed to size the
// panes control to fill the window.
// Removed several unnecessary variables.
// Cleaned up comments.
//
// Revision 1.7  2005/05/19 01:57:52  kenjen
// Added more menu items, reorganized menus
// and added some line divisions.
//
// Revision 1.6  2005/05/19 01:52:42  kenjen
// More cleanup.  Eliminate some redundant things.  After
// being enlightened about the popup shell unique naming
// behavior for menus, the menus now have their own names, 
// and the parent menuButtons are correctly attached to
// the popup shell.  Also, I took out some short Args
// setups and folded them into widget creation using
// XtVaCreate...() parameters.  It shortens the code
// somewhat and makes it look a little more tidy.
//
// Revision 1.5  2005/05/19 01:46:35  kenjen
// Working on splitting the menu bar and the main
// program area by a pane control.
//
// Revision 1.4  2005/05/19 01:41:46  kenjen
// Dropdown menus are working again.
// It seems the XtCreatePopUpShell uses a default name
// "menu" to identify the popup menu unless you change
// it specifically.  So, being "friendly" by uniquely
// naming each menu means you also have to identify
// that menu to the popup shell.
//
// Revision 1.3  2005/05/19 01:34:43  kenjen
// More cleanup and experimentation.
// Different layout with boxes around things, but
// I broke the pop-up menus.
//
// Revision 1.2  2005/05/19 01:27:59  kenjen
// Tidy up and make things readable.
//
// Revision 1.1  2005/05/19 01:18:20  kenjen
// Initial revision
//

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Sme.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

void quit (void);

void do_menu_selection (Widget item_selected,
			XtPointer unused1, XtPointer unused2);

int nDebug = 1;

XtAppContext application_context;

Widget hMainShell;
Widget hMainRowMenu;
Display *TheDisplay;

int TheScreen;
int nTextScreen;
int nGraphicsScreen;
int NumScrns;
int root_window;

int screen_width;
int screen_height;

unsigned long white_pixel;
unsigned long black_pixel;
Colormap DefaultColorMap;
int NumDisplayCells;

int ScreenDepth;
Visual *ScrnVisual;

Dimension MainX;
Dimension MainY;
Dimension MainWidth;
Dimension MainHeight;

char *MenuText[200];
int MenuID[200];
int NumMenu;



void
do_menu_selection (Widget item_selected, XtPointer unused1, XtPointer unused2)
{
  char *name;

  name = XtName (item_selected);
  printf ("Menu item `%s' has been selected.\n", name ? name : "NULL");
  if (name && !strcmp (name, "Quit"))
    quit ();
}


void
quit (void)
{
  XtDestroyApplicationContext (application_context);
  exit (0);
}

int
main (int argc, char **argv)
{
  Widget hWindowPanel;
  Widget hBoxPanel;
  Widget hPanePanel;
  Widget hMenuBar;
  Widget hProgramPanel;
  Widget MenuButton[20];
  Widget SubMenu[20];
  Widget Menu_element;
  Arg args[10];
  int i;
  int NumArgs;
  char szTemp[32];
  int ID = 0;
  int NumMenu = 0;
  int TopMenuID = -1;
  int LowerMenuID = -1;


  XtToolkitInitialize ();

  application_context = XtCreateApplicationContext ();

  TheDisplay = XtOpenDisplay (application_context,
			      "", "myprog", "MyProg", NULL, 0, &argc, argv);

//      The top top top level "Shell" is the window itself

  hMainShell = XtVaAppCreateShell ("myprog",
				   "MyProg",
				   applicationShellWidgetClass,
				   TheDisplay, NULL);


//      Determine X Window setup:
  TheScreen = DefaultScreen (TheDisplay);

  root_window = RootWindow (TheDisplay, TheScreen);
  screen_width = XDisplayWidth (TheDisplay, TheScreen);
  screen_height = XDisplayHeight (TheDisplay, TheScreen);
  white_pixel = WhitePixel (TheDisplay, TheScreen);
  black_pixel = BlackPixel (TheDisplay, TheScreen);
  DefaultColorMap = XDefaultColormap (TheDisplay, TheScreen);
  NumDisplayCells = XDisplayCells (TheDisplay, TheScreen);
  ScreenDepth = XDefaultDepth (TheDisplay, TheScreen);
  ScrnVisual = XDefaultVisual (TheDisplay, TheScreen);
  NumScrns = XScreenCount (TheDisplay);
  nTextScreen = TheScreen;
  nGraphicsScreen = TheScreen;

  if (NumScrns > 1)
    nGraphicsScreen = TheScreen + 1;

//      Calcuate desired main window geometry
  MainX = screen_width / 10;
  MainY = screen_height / 10;
  MainWidth = screen_width - (MainX * 2);
  MainHeight = screen_height - (MainY * 3);

  if (nDebug)
    {
      printf ("      Number of screens: %d\n", NumScrns);
      printf ("            Text screen: screen %d\n", nTextScreen);
      printf ("        Graphics screen: screen %d\n", nGraphicsScreen);
      printf ("                 Screen: %d x %d\n", screen_width,
	      screen_height);
      printf ("                  Depth: %d\n", ScreenDepth);
      printf ("Number of Display Cells: %d\n", NumDisplayCells);
      printf ("            Visual mode: %s\n",
	      ScrnVisual ? ScrnVisual : (Visual *) "NULL");
      printf ("            Window size: %d x %d\n", MainWidth, MainHeight);
      printf ("          Window corner: (%d, %d)\n", MainX, MainY);
    }

//      Resize main window:
  NumArgs = 0;
  XtSetArg (args[NumArgs], XtNx, MainX);
  NumArgs++;
  XtSetArg (args[NumArgs], XtNy, MainY);
  NumArgs++;
  XtSetArg (args[NumArgs], XtNwidth, MainWidth);
  NumArgs++;
  XtSetArg (args[NumArgs], XtNheight, MainHeight);
  NumArgs++;
  XtSetValues (hMainShell, args, NumArgs);

//      "Form" gives us a container for widgets that sizes itself
//      large enough to contain all its children.

  hWindowPanel = XtVaCreateManagedWidget ("mainPanel",
					  formWidgetClass, hMainShell, NULL);

// "Pane" will give us separate working areas that we will
//      use as a "menubar" on top, and the main display at 
//  the bottom.  Here we also force the pane control to the
//      same size of the window.

  hPanePanel = XtVaCreateManagedWidget ("panePanel",
					panedWidgetClass,
					hWindowPanel,
					XtNwidth, MainWidth,
					XtNheight, MainHeight, NULL);

//      The "wrapper" boxes for the menu bar, and the program area 
//      appears to be absorbed by the pane.  The boxes do not appear
//      to render.  This menuBox is needed to "contain" the menu 
//      buttons in a single pane.

  hMenuBar = XtVaCreateManagedWidget ("menuBox",
				      boxWidgetClass, hPanePanel, NULL);

//      Put something in the lower division of the pane.

  hProgramPanel = XtVaCreateManagedWidget ("programBox",
					   boxWidgetClass, hPanePanel, NULL);

  XtVaCreateManagedWidget ("Your Program Here",
			   labelWidgetClass, hProgramPanel, NULL);

//      Add "File" menu to the menu bar.

  MenuButton[++TopMenuID] = XtVaCreateManagedWidget ("File",
						     menuButtonWidgetClass,
						     hMenuBar,
						     XtNmenuName, "FileMenu",
						     NULL);

  SubMenu[++LowerMenuID] = XtCreatePopupShell ("FileMenu",
					       simpleMenuWidgetClass,
					       MenuButton[TopMenuID],
					       NULL, 0);

  Menu_element = XtCreateManagedWidget ("Save",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

  Menu_element = XtCreateManagedWidget ("Save As...",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

  Menu_element = XtCreateManagedWidget ("line",
					smeLineObjectClass,
					SubMenu[LowerMenuID], NULL, 0);

  Menu_element = XtCreateManagedWidget ("Quit",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

// Add "Edit" menu to the menu bar

  MenuButton[++TopMenuID] = XtVaCreateManagedWidget ("Edit",
						     menuButtonWidgetClass,
						     hMenuBar,
						     XtNmenuName, "EditMenu",
						     NULL);

  SubMenu[++LowerMenuID] = XtCreatePopupShell ("EditMenu",
					       simpleMenuWidgetClass,
					       MenuButton[TopMenuID],
					       NULL, 0);

  Menu_element = XtCreateManagedWidget ("Text",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

  Menu_element = XtCreateManagedWidget ("line",
					smeLineObjectClass,
					SubMenu[LowerMenuID], NULL, 0);

  Menu_element = XtCreateManagedWidget ("Cut",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

  Menu_element = XtCreateManagedWidget ("Copy",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

  Menu_element = XtCreateManagedWidget ("Paste",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

  Menu_element = XtCreateManagedWidget ("line",
					smeLineObjectClass,
					SubMenu[LowerMenuID], NULL, 0);

  Menu_element = XtCreateManagedWidget ("Delete",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

// Add "Help" menu to the menu bar

  MenuButton[++TopMenuID] = XtVaCreateManagedWidget ("Help",
						     menuButtonWidgetClass,
						     hMenuBar,
						     XtNmenuName, "HelpMenu",
						     NULL);
  SubMenu[++LowerMenuID] = XtCreatePopupShell ("HelpMenu",
					       simpleMenuWidgetClass,
					       MenuButton[TopMenuID],
					       NULL, 0);
  Menu_element = XtCreateManagedWidget ("Help",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

  Menu_element = XtCreateManagedWidget ("line",
					smeLineObjectClass,
					SubMenu[LowerMenuID], NULL, 0);

  Menu_element = XtCreateManagedWidget ("About",
					smeBSBObjectClass,
					SubMenu[LowerMenuID], NULL, 0);
  XtAddCallback (Menu_element, XtNcallback, do_menu_selection, NULL);

//      Fire Up the GUI

  XtRealizeWidget (hMainShell);

  XtAppMainLoop (application_context);

  return 0;
}
